"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const os_1 = require("os");
const path = require("path");
const stack_1 = require("./stack");
const imports_1 = require("./imports");
const api_1 = require("./api");
let id = 0;
class Fibra {
    constructor(task, options = {}) {
        this.task = task;
        this.id = ++id;
        const caller = stack_1.getCaller();
        this.fileName = `${caller}.fibra${this.id}`;
        this.dirName = path.dirname(caller);
        this.import = options.import;
        this.api = options.api;
    }
    generateFs() {
        return `var __dirname = ${JSON.stringify(this.dirName)};\n`
            + `var __filename = module.filename = ${JSON.stringify(this.fileName)};\n`;
    }
    generateAux() {
        return `Error.prototype.toJSON = function () { return { message: this.message, name: this.name }; };\n`;
    }
    createProcess(code) {
        let resolve;
        let reject;
        const promise = new Promise((_resolve, _reject) => {
            resolve = _resolve;
            reject = _reject;
        });
        let result;
        let error;
        const args = ['--eval', code];
        const inspect = process.execArgv.find((item) => item.startsWith('--inspect'));
        if (inspect) {
            const [flag, masterPort] = inspect.split('=');
            const port = (+masterPort || 9229) + this.id;
            args.unshift(`${flag}=${port}`);
            console.log(`Starting fibra ${this.id} inspector on port ${port}.`);
        }
        this.child = child_process_1.spawn('node', args, {
            cwd: process.cwd(),
            env: process.env,
            stdio: ['inherit', 'inherit', 'inherit', 'ipc'],
        });
        this.child.on('message', (data) => {
            switch (data.type) {
                case 'exit':
                    result = data.result;
                    error = data.error;
                    break;
                case 'prop':
                    (async () => {
                        try {
                            const result = await data.path.reduce((obj, key) => obj[key], this.api);
                            this.child.send({ id: data.id, result });
                        }
                        catch (error) {
                            this.child.send({ id: data.id, error });
                        }
                    })();
                    break;
                case 'call':
                    (async () => {
                        try {
                            const fn = data.path.pop();
                            const obj = data.path.reduce((obj, key) => obj[key], this.api);
                            const result = await obj[fn].apply(obj, data.args);
                            this.child.send({ id: data.id, result });
                        }
                        catch (error) {
                            this.child.send({ id: data.id, error });
                        }
                    })();
                    break;
                default:
                    console.error(`Unknown action ${data.type}!`);
                    break;
            }
        });
        this.child.on('error', (error) => {
            reject(error);
        });
        this.child.on('exit', (code, signal) => {
            if (code === 0) {
                resolve(result);
            }
            else {
                reject(error || new Error(signal));
            }
        });
        return promise;
    }
    run(...args) {
        let code = `'use strict';\n`;
        code += this.generateFs();
        code += this.generateAux();
        const context = [];
        if (this.import) {
            const imports = imports_1.generateImports(this.import);
            context.push(`import: ${imports}`);
        }
        if (this.api) {
            const api = api_1.generateApi();
            context.push(`api: ${api}`);
        }
        code += `(${this.task.toString()}).apply({${context.join(', ')}}, ${JSON.stringify(args)})\n`;
        code += `.then(\n`;
        code += `  function (r) { process.send({ type: 'exit', result: r }); process.removeAllListeners(); },\n`;
        code += `  function (e) { process.send({ type: 'exit', error: e }); process.exit(1); }\n`;
        code += `);\n`;
        return this.createProcess(code);
    }
    kill() {
        if (this.child) {
            this.child.kill('SIGKILL');
        }
    }
}
Fibra.cores = os_1.cpus().length;
exports.default = Fibra;
//# sourceMappingURL=index.js.map